from funittest import logical
from funittest import dataprovider
from funittest import physical
from funittest import interpreter
from funittest import scenarios
from funittest import Schema
from funittest import models

class Validation(models.Test):
    """
    Test validation in KSS.
    
    1. First the user loggon
    
    2. The user create a document
    
    3. The user want to inline-edit the title of document
    
    4. The user change the title (delete it)
    
    5. The user click on save
    
    5a. The user press ok
    
    5b. The user click on cancel
    
    6. The user delete the document
    """
 
    schema = Schema({'field': dataprovider.kss.archetypefields,
                   'user': dataprovider.plone.user,
                   'content': dataprovider.plone.content})
    
    def setUp(self):
        self._user = dataprovider.plone.user.get("admin")
        self._content = dataprovider.plone.content.get("document-1")
        self._field = dataprovider.kss.archetypefields.get("title")
        
        self._edited_content = self._content.copy()
        self._edited_content[self._field['id']] = '' # By default try to use an empty title

    def tearDown(self):
        """
        I think we should do something like:
        if test is failed then 
            1. tell it :)
            2. go to a consistent state
            3. delete stuff
        else 
            1. delete stuff
            
        but since"going to a consistent state" depends on the kind of test, I need to do it in every test variation.
        """
        #logical.plone.navigation.top()
        #scenarios.plone.deletecontent(content=self._content)

    def step_1(self):
        scenarios.plone.login(user=self._user)
        logical.plone.navigation.top()
    
    def step_2(self):
        scenarios.plone.addcontent(content=self._content)

    def step_3(self):
        logical.kss.fields.select(self._field)
    
    def step_4(self):
        logical.kss.fields.fill(self._field, self._edited_content)
    
    def step_5(self):
        logical.kss.fields.validate(self._field, self._edited_content)

    def step_5a(self):
        logical.kss.fields.validateWithKey(self._field, self._edited_content)

    def step_5b(self):
        logical.kss.fields.cancel(self._field, self._content)

    def test(self):
        self.expect_ok('1','2','3','4','5')
        
    def test_4a(self):
        self._edited_content[self._field['id']] = 'Valid stuff'
        self.expect_ok('1','2','3','4','5')
        
    def test_5a(self):
        self.expect_ok('1','2','3','4','5a')
        
    def test_5b(self):
        self.expect_ok('1','2','3','4','5b')
