import os
import time
from funittest import models
from funittest import interpreter
from funittest.utils import resourceFile

class Content(models.DataProvider):

    def new(self, key):
        pass

    def establish(self, content):
        pass

    def get(self, key, unique_file_names = True):
        # In general, a unique file name is needed to avoid name collisions 
        # in Plone. 
        #
        # This happens when adding a file with the same filename as another
        # file in the same folder.
        #
        # If you want all files in a content to get a unique name, then
        # just call the get method as usual. 
        #
        # If you want to specifically test file name collisions, you can use 
        #
        # dataprovider.plone.content.get('mycontent', unique_file_names = False)
        #
        content = self.data[key].copy()
        if unique_file_names:
            for key , value in content.items():
                # All keys that end with _file are considered files
                if key.endswith('_file'):
                    head, tail = os.path.split(value)
                    content[key] = '/tmp/' + str(time.time()).replace('.','') + tail
                    if not interpreter.is_dry_run():
                        # No need for a symlink in dry run mode
                        os.symlink(value, content[key])
        return content

defaults = {'id': 'samplecontent',
            'title':"Document",
            'description':'This is an example document',
            'type_name':'Document',
            }
key='id'
content = Content("Plone", defaults, key)
provide = content.provide
provide({'id':'document-1',
         'title':'Document 1',
         'text':'Document 1 text',
         'type_name':'Document',
        }
       )

provide({'id':'front-page',
         'title':'Welcome to Plone',
         'description':'Congratulations! You have successfully installed Plone.',
         'type_name':'Document'
        }
       )

provide({'id':'page1',
         'title':'Page1',
         'type_name':'Document',
         'text':'Page 1 text',
        }
       )

provide({'id':'topic1',
         'title':'Topic1',
         'type_name':'Topic'
        }
       )

provide({'id':'event1',
         'title':'Event1',
         'description':'This is an example event',
         'type_name':'Event'
        }
       )

provide({'id':'file1',
         'title':'File1',
         'type_name':'File',
         'file_file': resourceFile("plone_wheel.png", egg='funittest.plone'),
        }
       )

provide({'id':'folder1',
         'title':'Folder1',
         'type_name':'Folder'
        }
       )

provide({'id':'image1',
         'title':'Image1',
         'image_file': resourceFile("plone_cookies.jpg", egg='funittest.plone'),
         'type_name':'Image'
        }
       )

provide({'id':'link1',
         'title':'Link1',
         'type_name':'Link'
        }
       )

provide({'id':'newsitem1',
         'title':'NewsItem1',
         'type_name':'News+Item',
         'text':'News Item 1 text',
        }
       )

provide({'id':'document1',
         'title':'Document1',
         'type_name':'Document',
         'text':'Document1 text',
        }
       )

provide({'id':'events',
         'title':'Events',
         'type_name':'Topic'
        }
       )

provide({'id':'news',
         'title':'News',
         'type_name':'Topic'
        }
       )

provide({'id':'searchcontent',
         'title':'Search Content',
         'type_name':'Document',
         'text':'the quick brown fox jumps over the lazy dog',
        }
       )







