from funittest import physical
from funittest import logical
from funittest import dataprovider
from funittest import interpreter
from funittest import models
from funittest import verify

class Navigation(models.Logical):

    @verify
    def down(self, content):
        """
        Navigate down to a content.
        """
        location = interpreter.get_location()
        context = {'id':location + content['id'],
                   'context': location + content['id'],
                   'title':content['title'],
                  }              
        interpreter.annotate("Navigation down to " + content['title'])
        if physical.plone.portaltab.has_content(content):
            physical.plone.portaltab.access(content)
        elif physical.plone.navigationtree.has_content(context):
            physical.plone.navigationtree.access(context)
        elif physical.plone.folderlisting.has_content(content):
            physical.plone.folderlisting.access(content)
        else:
            if physical.plone.foldercontents.has_tab():
                physical.plone.foldercontents.access()
                physical.plone.foldercontents.show_all()
                physical.plone.foldercontents.down(content)
            else:
                logical.plone.navigation.open_context(context)

    def up_using_breadcrumbs(self):
        breadcrumbs = physical.plone.breadcrumbs.get()
        if len(breadcrumbs) == 0:
            # No breadcrumbs, so just go to the top
            logical.plone.navigation.top()
        elif len(breadcrumbs) >= 1:
            physical.plone.breadcrumbs.last()
            
    up_composite_execution_behaviour = [up_using_breadcrumbs,]
    @verify
    def up(self):
        interpreter.annotate("Navigate up")
        
    def top_using_url(self):
        context = dataprovider.plone.context.get('root')
        logical.plone.navigation.open_context(context)

    top_composite_execution_behaviour = [top_using_url,]
    @verify
    def top(self):
        interpreter.annotate("Navigate to top")

    @verify
    def stepwise_down(self, items):
        for item in items:
            logical.plone.navigation.down(item)

    @verify
    def stepwise_up(self, items):
        for item in items:
            logical.plone.navigation.up()

    @verify
    def stepwise_down_and_up(self, items):
        logical.plone.navigation.stepwise_down(items)
        logical.plone.navigation.stepwise_up(items)

    @verify
    def stepwise_top_down_and_up(self, items):
        logical.plone.navigation.top()
        logical.plone.navigation.stepwise_down(items)
        logical.plone.navigation.stepwise_up(items)

    def open_context(self, context):
        interpreter.annotate("Navigate to a context using the navigation toolbar")
        physical.plone.navigation.open(context['context'])

    def open_path(self, path):
        interpreter.annotate("Navigate to a path using the navigation toolbar")
        url = ""
        for item in path['path']:
            url += '/' + item['id']
        physical.plone.navigation.open(url)
