from funittest import physical
from funittest import interpreter

from types import StringTypes
from types import DictType

from funittest import models

class Forms(models.Physical):

    def edit_content(self, content):
        forms = physical.plone.forms.getForms()
        form = forms.getForm('edit_form')
        
        values = []
        for key, value in content.items():           
            if key not in ['type_name', 'id']:
                values.append( {'id':key, 
                                'value':value} )
        form.fillForm(values)

    def getForms(self):
        return ClientForms(interpreter, self._getFormNames())    
        
    def getArchetypesFieldIds(self, form_locator):
        # Takes everything that has a class of "field Archetypes*"
        return self._getArchetypesFieldIds(form_locator)

    def _getFormNames(self):
        js_call = "getFormNames = function() {" +\
                  "    var elements = selenium.page().getDocument().getElementsByTagName('form');" +\
                  "    var result = '';" +\
                  "    for (var i = 0; i < elements.length; i++) {" +\
                  "        result += elements[i].name;" +\
                  "        result += ',';" +\
                  "    };" +\
                  "    return result;"+\
                  "};" +\
                  "getFormNames();"
        names = interpreter.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        names = names.split(',')
        if names:
            if names[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                names.pop()
        return names

    def _getArchetypesFieldIds(self, form_locator):
        # Get selected values
        js_call = "getFieldIds = function(locator) {" +\
                  "    var element = selenium.page().findElement(locator);" +\
                  "    var elements = element.getElementsByTagName('div');" +\
                  "    var result = '';" +\
                  "    for (var i = 0; i < elements.length; i++) {" +\
                  "        if (elements[i].id != '') {" +\
                  "            result += elements[i].id;" +\
                  "            result += ',';" +\
                  "         };" +\
                  "     };" +\
                  "     return result;" +\
                  "};" +\
                  "getFieldIds(\"%s\");" % form_locator
        ids = interpreter.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        ids = ids.split(',')
        if ids:
            if ids[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                ids.pop()
        # The names are taken from all DIV tags, incuding help text DIVs.
        # Let's filter out the archetypes fields
        field_ids = []
        for id in ids:
            if id.startswith('archetypes-fieldname-'):
                field_ids.append(id[len('archetypes-fieldname-'):])
        return field_ids
        

class ClientForms:

    def __init__(self, interpreter, form_names):
        interpreter = interpreter
        self.form_names = form_names
    
    def getFormNames(self):
        return self.form_names
    
    def getForm(self, name):
        """Get the form by name"""
        locator = "//form[@name='%s']" % name
        return ClientForm(interpreter, locator)

    def getFormByLocator(self, locator):
        return ClientForm(interpreter, locator)

class ClientForm:

    def __init__(self, interpreter, locator):
        interpreter = interpreter
        if not locator:
            raise "A valid form_name is needed"
        self.form_locator = locator

    def getFormLocator(self):
        return self.form_locator
        
    def getArchetypesFieldIds(self):
        return physical.plone.forms.getArchetypesFieldIds(self.getFormLocator())

    def fillForm(self, values):
        for field in values:
            widget = self.getWidget(field['id'])
            if widget is None and interpreter.is_dry_run():
                pass
            else:                    
                widget.fill(field['value'])

    def getWidgetName(self, field_id):
        """ For each field in a form, we need to know the widget that is
            used. Based on this information, we can then fill in the
            field automatically.
            
            For Archetypes, this means getting the attribute of the 
            div tag with id "archetypes-fieldname-title", which could
            be "field ArchetypesStringWidget".            
        """
        if interpreter.is_element_present("//div[@id='archetypes-fieldname-%s']/@class" % field_id):
            attribute = interpreter.get_attribute("//div[@id='archetypes-fieldname-%s']/@class" % field_id)
            widget_name=""
            for part in attribute.split(" "):
                if part.startswith("Archetypes"):
                    widget_name = part
            if widget_name in ["ArchetypesStringWidget",
                               "ArchetypesTextAreaWidget",
                               "ArchetypesRichWidget",
                               "ArchetypesIntegerWidget",
                               "ArchetypesBooleanWidget",
                               "ArchetypesLinesWidget",
                               "ArchetypesSelectionWidget",
                               "ArchetypesMultiSelectionWidget",
                               "ArchetypesCalendarWidget",
                               "ArchetypesFileWidget",
                               "ArchetypesImageWidget",
                               "ArchetypesReferenceWidget",
                               "ArchetypesInAndOutWidget",
                               "ArchetypesPicklistWidget",
                              ]:
                pass
            else:
                widget_name = "BaseWidget"
            return widget_name
        else:
            return "NonArchetypesWidget"

    def getWidget(self, field_id):
        widget_name = self.getWidgetName(field_id)
        form_locator = self.getFormLocator()
        if widget_name == "BaseWidget":
            return BaseWidget(interpreter, form_locator, field_id)
        elif widget_name == "NonArchetypesWidget":
            widget = NonArchetypesWidget(interpreter, form_locator, field_id)
            if widget.probe():
                return widget
            widget = NonArchetypesTextAreaWidget(interpreter, form_locator, field_id)
            if widget.probe():
                return widget
            widget = NonArchetypesSelectionWidget(interpreter, form_locator, field_id)
            if widget.probe():
                return widget
        if widget_name == "ArchetypesStringWidget":
            return ArchetypesStringWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesTextAreaWidget":        
            return ArchetypesTextAreaWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesRichWidget":        
            return ArchetypesRichWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesIntegerWidget":        
            return ArchetypesIntegerWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesBooleanWidget":        
            return ArchetypesBooleanWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesLinesWidget":        
            return ArchetypesLinesWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesSelectionWidget":        
            # There are two possibilities for the selection widget: Select or Radio
            widget = ArchetypesSelectionWidget(interpreter, form_locator, field_id)
            if not widget.probe():
                # If the widget can not be confirmed to be a selection widget, we deduce that it should be a radio selection widget
                widget = ArchetypesSelectionWidgetRadio(interpreter, form_locator, field_id)
            return widget
        elif widget_name == "ArchetypesMultiSelectionWidget":        
            # There are two possibilities for the selection widget: Select or Checkbox
            widget = ArchetypesMultiSelectionWidget(interpreter, form_locator, field_id)
            if not widget.probe():
                # If the widget can not be confirmed to be a selection widget, we deduce that it should be a radio selection widget
                widget = ArchetypesMultiSelectionWidgetCheckbox(interpreter, form_locator, field_id)
            return widget
        elif widget_name == "ArchetypesCalendarWidget":        
            return ArchetypesCalendarWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesFileWidget":        
            return ArchetypesFileWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesImageWidget":        
            return ArchetypesImageWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesReferenceWidget":        
            return ArchetypesReferenceWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesInAndOutWidget":        
            return ArchetypesInAndOutWidget(interpreter, form_locator, field_id)
        elif widget_name == "ArchetypesPicklistWidget":        
            return ArchetypesPicklistWidget(interpreter, form_locator, field_id)

class BaseWidget:
    
    def __init__(self, interpreter, form_locator, field_id):
        interpreter = interpreter
        self.form_locator = form_locator
        self.field_id = field_id

    def probe(self):
        """Probe the widget to see whether the control is present"""
        return interpreter.is_element_present(self.getLocator())

    def getValue(self):
        """Return the current value the filled in field.
        """
        raise "Not implemented"
        
    def getForm(self):
        return self.form_locator
        
    def getField(self):
        return "//div[@id='archetypes-fieldname-%s']" % self.field_id
        
    def getControl(self):
        return "//input[@name='%s']" % self.field_id

    def getFieldLocator(self):        
        return self.getForm() + self.getField()
        
    def getLocator(self):
        return self.getForm() + self.getField() + self.getControl()

    def fill(self, value):
        """Fill in the widget with the given value"""
        MAXCHARS=2000
        if len(value)>MAXCHARS:
            # In order to avoid a 414 "Request URI Too Large" error, we need to send text in chunks
            # SeleniumRC does not allow POST requests, and there is a limit on the size of GET requests
            # This code will become obsolete as soon as a new version of SeleniumRC reponds with
            # true to the isPostSupported command.
            interpreter.type(self.getLocator(), value[:MAXCHARS])
            value = value[MAXCHARS:]
            while value:
                js_call = "addToTextArea = function(locator, text) { " +\
                          "var element = selenium.page().findElement(locator);" +\
                          "element.innerHTML+=text;" +\
                          "};" +\
                          "addToTextArea(\"%s\",\"%s\")" % (self.getLocator(), value[:MAXCHARS].replace(u'"',u"'"))
                # The type command does not allow appending text, so we have to inject
                # the text using our own method. The use of the type_keys command is too slow and not 
                # reliable enough for large texts. 
                interpreter.get_eval(js_call)
                value = value[MAXCHARS:]
        else:
            if self.probe():
                interpreter.type(self.getLocator(), value)
            else:
                interpreter.type(self.field_id, value)

class NonArchetypesWidget(BaseWidget):
    
    def probe(self):
        """Probe the widget to see whether the control is present"""
        return interpreter.is_element_present(self.getLocator())

    def getValue(self):
        """Return the current value the filled in field.
        """
        raise "Not implemented"
        
    def getField(self):
        return ''
        
    def getControl(self):
        return "//input[@name='%s']" % self.field_id

    def getFieldLocator(self):        
        return self.getForm() + self.getField()
        
    def getLocator(self):
        return self.getForm() + self.getField() + self.getControl()

    def fill(self, value):
        """Fill in the widget with the given value"""
        if type(value)==type(True):
            if value:
                interpreter.check(self.getLocator())
            else:
                interpreter.uncheck(self.getLocator())
        else:
            if interpreter.is_element_present(self.getLocator()):
                interpreter.type(self.getLocator(), value)

class NonArchetypesTextAreaWidget(NonArchetypesWidget):

    def getControl(self):
        return "//textarea[@name='%s']" % self.field_id

class NonArchetypesSelectionWidget(NonArchetypesWidget):
    def getControl(self):
        return "//select[@name='%s']" % self.field_id

    def getValue(self):
        """
        Read the currently set value
        """        
        # Get selected values
        return interpreter.get_selected_values(self.getControl())

    def fill(self, value):
        """Fill in the widget with the given value"""
        interpreter.select(self.getLocator(),"value=%s" % value)

class ArchetypesStringWidget(BaseWidget):
    def getControl(self):
        return "//input[@name='%s']" % self.field_id
    
class ArchetypesTextAreaWidget(BaseWidget):
    def getControl(self):
        return "//textarea[@name='%s']" % self.field_id

class ArchetypesRichWidget(BaseWidget):
    def getControl(self):
        return "//textarea[@name='%s']" % self.field_id

class ArchetypesIntegerWidget(BaseWidget):
    def getControl(self):
        return "//input[@name='%s']" % self.field_id

class ArchetypesBooleanWidget(BaseWidget):
    def getControl(self):
        return "//input[@name='%s_visible']" % self.field_id

    def fill(self, value):
        """Fill in the widget with the given value"""
        
        # Get the current value
        current_value = interpreter.get_value(self.getLocator())
        if value:
            if current_value=='off':
                interpreter.click(self.getLocator())
            else:
                pass
        else:
            if current_value=='on':
                interpreter.click(self.getLocator())
            else:
                pass
        
class ArchetypesLinesWidget(BaseWidget):
    def getControl(self):
        return "//textarea[@name='%s:lines']" % self.field_id

    def getField(self):
        return "//div[@id='archetypes-fieldname-%s']" % self.field_id

    def fill(self, value):
        """Fill in the widget with the given value"""
        if type(value) in StringTypes:
            value=[value]
        interpreter.type(self.getLocator(), '\n'.join(value))
        
class ArchetypesSelectionWidget(BaseWidget):
    
    def getControl(self):
        return "//select[@name='%s']" % self.field_id

    def getValue(self):
        """
        Read the currently set value
        """        
        # Get selected values
        return interpreter.get_selected_values(self.getControl())
        
    def fill(self, value):
        """Fill in the widget with the given value"""
        interpreter.select(self.getLocator(),"value=%s" % value)

class ArchetypesSelectionWidgetRadio(BaseWidget):

    def getControl(self):
        return "//input[@name='%s']" % self.field_id
        
    def getValue(self):
        """
        Read the currently set value
        """        
        return physical.plone.utils.getAllCheckedRadioFieldsById(self.getFieldLocator())
        
    def fill(self, value):
        """Fill in the widget with the given value"""
        locator = self.getFieldLocator() + "//input[@value='%s']" % value
        interpreter.check(locator)

class ArchetypesMultiSelectionWidget(BaseWidget):
    
    def getControl(self):
        return "//select[@name='%s:list']" % self.field_id

    def probe(self):
        """Probe the widget to see whether the control is present"""
        return interpreter.is_element_present(self.getLocator() + "//@multiple")

    def getValue(self):
        """
        Read the currently set value
        """        
        if interpreter.is_something_selected(self.getLocator()):
            # Get selected values
            selected = interpreter.get_selected_values(self.getLocator())
        else:
            selected = []
        return selected
        
    def fill(self, value):
        """Fill in the widget with the given value"""                   
        if type(value) in StringTypes:
            value=[value]
        current_selections = self.getValue()
        # First remove all existing selections if they are not to be set anyway
        for selection in current_selections:
            if not selection in value:
                interpreter.remove_selection(self.getLocator(),"value=%s" % selection)        

        # Make missing selections                
        for selection in value:
            # No need to select what was already selected
            if not selection in current_selections:
                interpreter.add_selection(self.getLocator(),"value=%s" % selection)

        #os.system("pause")

 
class ArchetypesMultiSelectionWidgetCheckbox(BaseWidget):
    def getControl(self):
        return "//input[@name='%s:list']" % self.field_id

    def getValue(self):
        """
        Read the currently set value
        """        
        return physical.plone.utils.getAllSelectedCheckboxFieldsById(self.getFieldLocator())
        
    def fill(self, value):
        """Fill in the widget with the given value"""                   
        
        if type(value) in StringTypes:
            value=[value]
        
        currently_checked=physical.plone.utils.getAllSelectedCheckboxFieldsById(self.getFieldLocator())
        currently_checked=currently_checked.values()

        # First remove all existing checks if they are not to be set anyway
        for check in currently_checked:
            if not check in value:
                #os.system("pause")
                interpreter.uncheck(self.getFieldLocator() + "//input[@value='%s']" % check)

        # Make missing selections                
        for check in value:
            # No need to check what was already checked
            if not check in currently_checked:
                #os.system("pause")
                interpreter.check(self.getFieldLocator() + "//input[@value='%s']" % check)
                #os.system("pause")
                
        #os.system("pause")

                
class ArchetypesCalendarWidget(BaseWidget):
    def getControl(self):
        return ""

    def getValue(self):
        """
        Read the currently set value
        returns a dictionary of the form
           {'year':2006,
            'month:2,
            'day':1,            
            'hour':12,
            'minute':30}
        """        
        value = {}
        for dc in ['year','month','day','hour','minute']:
            locator = self.getFieldLocator() + "//select[@name='datewidget_%s']" % dc
            value[dc]=int(interpreter.get_selected_values(locator))
        return value
        
    def fill(self, value):
        """Fill in the widget with the given value
           Expecting either a DateTime objet or a dictionary of the form
           {'year':2006,
            'month:2,
            'day':1,            
            'hour':12,
            'minute':30}
            You can leave out any of the keys. Date components that
            are not specified are left as is in the form.
        """       
        if type(value) == DictType:
            pass
        else:
            # Expecting DateTime object
            parts = value.parts()[:5]
            value = {'year':parts[0],
                     'month':parts[1],
                     'day':parts[2],            
                     'hour':parts[3],
                     'minute':parts[4]}
        
        # Minutes can only be filled in as multiple of 5
        if value['minute']:
            value['minute'] = int(value['minute']) - int(value['minute']) % 5
                     
        for dc in ['year','month','day','hour','minute']:
            if dc in value.keys():
                # %02s allows making "02" out of "2" and "2006" out of 2006
                locator = self.getFieldLocator() + "//select[@name='%s_%s']" % (self.field_id, str(dc))
                interpreter.select(locator, "value=%02d" % int(value[dc]))
        
class ArchetypesFileWidget(BaseWidget):
    def getControl(self):
        return "//input[@name='filewidget_file']"

    def fill(self, value):
        """Fill in the widget with the given value"""
        interpreter.type(self.getLocator(), value)
        
class ArchetypesImageWidget(BaseWidget):
    def getControl(self):
        return "//textarea[@name='%s']" % self.field_id

    def fill(self, value):
        """Fill in the widget with the given value"""
        interpreter.type(self.getLocator(), value)
        
class ArchetypesReferenceWidget(BaseWidget):
    def getControl(self):
        return "//textarea[@name='%s']" % self.field_id

    def fill(self, value):
        """Fill in the widget with the given value"""
        interpreter.type(self.getLocator(), value)
        
class ListToListWidget(BaseWidget):        
    def getControl(self):
        return ""

    def getValue(self):
        """
        Read the currently set value
        """
        locator = self.getDestinationControl()
        if interpreter.is_something_selected(locator):
            # Get selected values
            selected = interpreter.get_selected_values(self.getDestinationControl())
        else:
            selected = []
        return selected
        
    def fill(self, value):
        """Fill in the widget with the given value"""

        if type(value) in StringTypes:
            value=[value]

        current_selections = self.getValue()    
                
        # First remove all existing selections if they are not to be set anyway
        for selection in current_selections:
            if not selection in value:
                interpreter.select(self.getDestinationControl(),"value=%s" % selection)
                interpreter.click(self.getRemoveControl())
                
        # Make missing selections                
        for selection in value:
            # No need to select what was already selected
            if not selection in current_selections:
                interpreter.add_selection(self.getSourceControl(),"value=%s" % selection)
                interpreter.click(self.getAddControl())

class ArchetypesInAndOutWidget(ListToListWidget):
    
    def getSourceControl(self):
        return self.getLocator() + "//select[@name='%s_options']" % self.field_id
        
    def getDestinationControl(self):        
        return self.getLocator() + "//select[@name='%s:list']" % self.field_id

    def getRemoveControl(self):
        return self.getLocator() + "//input[@value='<<']"

    def getAddControl(self):
        return self.getLocator() + "//input[@value='>>']"        

class ArchetypesPicklistWidget(ListToListWidget):
        
    def getSourceControl(self):
        return self.getLocator() +  "//select[@name='%s_options']" % self.field_id
        
    def getDestinationControl(self):        
        return self.getLocator() + "//select[@name='%s:list']" % self.field_id

    def getRemoveControl(self):
        return self.getLocator() + "//input[@value='<<']"

    def getAddControl(self):
        return self.getLocator() + "//input[@value='>>']"
        
