from funittest import interpreter
from funittest import models

class Utils(models.Physical):

    __registered_getIdsOfTagWithType__=0

    def getHTMLSource(self):
        """ Get the HTML Source from the browser, making sure that the
            code is surrounded by proper HTML tags.
        """
        location = interpreter.get_location()
        if not location:
            return None
        elif location=='about:blank':
            physical.plone.navigation.open('')
            location = interpreter.get_location()

        html = "<html>"
        html = html + interpreter.get_html_source()
        html = html + "</html>"        

        return html

    def _get_hrefs(self, locator, tag):
        # Get selected values
        js_call = "getIdsOfTagWithTypeByElement = function(element, tag) {" +\
                  "var elements = element.getElementsByTagName(tag);" +\
                  "var result = '';" +\
                  "for (var i = 0; i < elements.length; i++) {" +\
                  "    result += elements[i].href;" +\
                  "        result += ',';" +\
                  "}" +\
                  "return result;" +\
                  "};" +\
                  "getIdsOfTagWithType = function(locator, tag) { " +\
                  "var element = selenium.page().findElement(locator);" +\
                  "return getIdsOfTagWithTypeByElement(element, tag);" +\
                  "};" +\
                  "getIdsOfTagWithType(\"%s\",\"%s\")" % (locator, tag)
        hrefs = interpreter.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        hrefs = hrefs.split(',')
        if hrefs:
            if hrefs[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                hrefs.pop()
        return hrefs

    def getHrefs(self, locator):
        """
        Given a locator, return the hrefs of all links.
        """
        return self._get_hrefs(locator, "a")

    def _get_field_ids(self, field_locator, input_type):
        # Get selected values
        js_call = "getIdsOfTagWithTypeByElement = function(element, tag, type) {" +\
                  "var elements = element.getElementsByTagName(tag);" +\
                  "var result = '';" +\
                  "for (var i = 0; i < elements.length; i++) {" +\
                  "    if (elements[i].type == type) {" +\
                  "        result += elements[i].id;" +\
                  "        result += ',';" +\
                  "    }" +\
                  "}" +\
                  "return result;" +\
                  "};" +\
                  "getIdsOfTagWithType = function(locator, tag, type) { " +\
                  "var element = selenium.page().findElement(locator);" +\
                  "return getIdsOfTagWithTypeByElement(element, tag, type);" +\
                  "};" +\
                  "getIdsOfTagWithType(\"%s\",\"%s\",\"%s\")" % (field_locator, "input", input_type)
        ids = interpreter.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        ids = ids.split(',')
        if ids:
            if ids[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                ids.pop()
        return ids                
        
    def getAllRadioFieldIds(self, field_locator):
        """
        Given a locator pointing to a field, and an input type,
        return the ids of all radio buttons.
        """
        return self._get_field_ids(field_locator, "radio")

    def getAllCheckboxFieldIds(self, field_locator):
        """
        Given a locator pointing to a field, and an input type,
        return the ids of all checkboxes.
        """
        return self._get_field_ids(field_locator, "checkbox")
        
    def getAllCheckedRadioFieldsById(self, field_locator):
        """ 
        Given a locator pointing to a field, return a dictionary mapping
        ids to values of checked radio buttons.
        """
        checked_by_id={}
        for id in self.getAllRadioFieldIds(field_locator):
            if interpreter.get_value(field_locator + "/input[@id='%s']" % id)=="on":            
                # We still need to know the value of the radio button, and fetch it by the id.
                value_control = "/input[@id='%s']/@value" % id
                field_id = field_locator + value_control
                if not interpreter.is_element_present(field_id):
                    raise "Can't find radio buttons with id \"%s\"" % field_id
                value = interpreter.get_attribute(field_locator + value_control)
                checked_by_id[id]=value
        return checked_by_id

    def getAllSelectedCheckboxFieldsById(self, field_locator):
        """ 
        Given a locator pointing to a field, return a dictionary mapping
        ids to values of selected checkboxes.
        """
        selected_by_id={}
        for id in self.getAllCheckboxFieldIds(field_locator):
            if interpreter.get_value(field_locator + "//input[@id='%s']" % id)=="on":
                # We still need to know the value of the checkbox, and fetch it by the id.
                value_control = "//input[@id='%s']/@value" % id
                field_id = field_locator + value_control
                if not interpreter.is_element_present(field_id):
                    raise "Can't find check boxes with id \"%s\"" % field_id
                value = interpreter.get_attribute(field_locator + value_control)
                selected_by_id[id]=value
        return selected_by_id

    def _get_field_values(self, field_locator, input_type, input_name):
        # Get selected values
        js_call = "getValuesOfTagWithTypeByElement = function(element, tag, type,name) {" +\
                  "var elements = element.getElementsByTagName(tag);" +\
                  "var result = '';" +\
                  "for (var i = 0; i < elements.length; i++) {" +\
                  "    if (elements[i].type == type) {" +\
                  "        if (elements[i].name == name) {" +\
                  "            result += elements[i].value;" +\
                  "            result += ',';" +\
                  "        }" +\
                  "    }" +\
                  "}" +\
                  "return result;" +\
                  "};" +\
                  "getValuesOfTagWithType = function(locator, tag, type, name) { " +\
                  "var element = selenium.page().findElement(locator);" +\
                  "return getValuesOfTagWithTypeByElement(element, tag, type,name);" +\
                  "};" +\
                  "getValuesOfTagWithType(\"%s\",\"%s\",\"%s\",\"%s\")" % (field_locator, "input", input_type, input_name)
        values = interpreter.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        values = values.split(',')
        if values:
            if values[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                values.pop()
        return values

    def getAllCheckboxFieldValues(self, field_locator, input_name):
        """
        Given a locator pointing to a field,
        return the values of all checkboxes.
        """
        return self._get_field_values(field_locator, "checkbox", input_name)

    def getAllSelectedCheckboxFieldValues(self, field_locator, input_name):
        """ 
        Given a locator pointing to a field, return a list of
        selected values of input fields with the given input_name
        """
        selected = []
        for value in self.getAllCheckboxFieldValues(field_locator, input_name):
            if interpreter.get_value(field_locator + "//input[@value='%s']" % value)=="on":
                selected.append(value)
        return selected
        
