# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from funittest import physical
from funittest import logical
from funittest import Testable
from funittest import interpreter

class Content(Testable):

    def existing_content(self, content):        
        """
        In the current context, create the content that is passed in like this:
        
        [{'id':'doc1', 'title':'Document 1', 'text':'Document 1 text', 'type_name':'Document'},
         {'id':'doc2', 'title':'Document 2', 'text':'Document 2 text', 'type_name':'Document'},
        ]
        Expecting to be logged in with a user that can add the content
        """
        interpreter.annotate("Setup content in a context.")
        for item in content:
            # Remember where we are before trying to go down
            breadcrumbs = physical.plone.breadcrumbs.get()
            # Try accessing the item
            logical.plone.navigation.down(item)
            # If the location hasn't changed, we need to add the item
            if physical.plone.breadcrumbs.get() == breadcrumbs:
                logical.plone.folder.add(item)
                logical.plone.content.edit(item)
                logical.plone.content.save(item)
                logical.plone.navigation.up()
            else:
                logical.plone.navigation.up()

    def merge_descending_path_top_down(self, descending_path):
        """
        In the current context, merge the path descending from the contex.

        Given a descending_path like this:

        [{'id':'folder1', 'title':'Folder 1', 'type_name':'Folder'},
         {'id':'folder2', 'title':'Folder 2', 'type_name':'Folder'},
         {'id':'doc2', 'title':'Document 2', 'text':'Document 2 text', 'type_name':'Document'},
        ]

        Expecting to be logged in with a user that can add the content.
        If an item on the path exists, it is just followed.
        If an item on the path does not exist, it is created.
        """
        logical.plone.navigation.top()
        for item in descending_path:
            # Remember where we are before trying to go down
            breadcrumbs = physical.plone.breadcrumbs.get()
            # Try accessing the item
            logical.plone.navigation.down(item)
            # If the location hasn't changed, we need to add the item
            if physical.plone.breadcrumbs.get() == breadcrumbs:
                logical.plone.folder.add(item)
                logical.plone.content.edit(item)
                logical.plone.content.save(item)
               
