from funittest import dataprovider
from funittest import logical
from funittest import models
from funittest import Schema

class Login(models.Scenario):
    """
    Login User
    
    Precondition: User must be registered
    
    1. Access a page with a login form
    2. Login
    3. Continue to the Portal home page
    """
    
    schema = Schema({"user" : dataprovider.plone.user})
    
    def step_1(self):
        logical.plone.application.access_login(self._user)

    def step_2(self):
        logical.plone.application.login(self._user)

    def step_3(self):
        logical.plone.navigation.top()

    def scenario(self):
        """
        User logs in
        """
        if self._user['password']==None:
            self.expect_ko(1,2)
            self.expect_ok(3)
        else:
            self.expect_ok(1,2,3)

    def scenario_2a(self):
        """
        User enters the wrong password
        """
        password = self._user['password']
        self._user['password']='wrongpassword'
        self.expect_ko(1,2)
        # Recover from the error by filling in the correct password this time
        self._user['password']=password
        self.expect_ok(2,3)

    def scenario_2b(self):
       """
       User is forced to change the password, because the password is 
       expired.
       """
       # Change must_change_password manually for a user or globally
       # in portal_memberdata
       # (There is no way to do this in a configlet right now)
       # 1. Login with the user
       # 2. Change the password
       pass

