from urlparse import urljoin

from funittest import dataprovider
from funittest import scenarios
from funittest import models
from funittest import Schema

class NoAccessTo(models.Scenario):
    """
    The site administrator does not want the current user to have access
    to certain url extensions of content.
    
    Preparation:
    
    1. Choose a user
    2. Choose a context
    3. Choose the list of templates that should not be accessible
    
    Use Case:
    
    1. Navigate to the given templates of the context
    """

    schema = Schema({"user":dataprovider.plone.user,
                     "context":dataprovider.plone.context,
                     "urlextension":dataprovider.plone.urlextension})

    def setUp(self):
        scenarios.plone.login._user = self._user
        scenarios.plone.login(user=self._user)

    def step_1(self):
        scenarios.plone.navigateto(context = self._context)

    def scenario(self):
        """
        Navigate to a context to make sure the user can not access it.
        """
        context = self._context.copy()
        for urlextension in self._urlextension['urlextension']:
            self._context['context'] = urljoin(context['context'], urlextension)
            self.expect_ok(1)

