from funittest import dataprovider
from funittest import logical
from funittest import models
from funittest import Schema

class Register(models.Scenario):
    """
    Registration of a new user to the Plone site
    
    Precondition
    
    1. Registration is allowed and user can choose password
    2. User is not yet registered
    
    Main scenario
    
    1. Access the registration form
    2. Fill in the registration form
    3. Submit the registration form
    4. Directly log in to the site
    """

    schema = Schema({"user":dataprovider.plone.user})

    def setUp(self):
        pass

    def step_1(self):
        logical.plone.register.access(self._user)
    
    def step_2(self):
        logical.plone.register.fill(self._user)

    def step_3(self):
        logical.plone.register.submit(self._user)

    def step_4(self):
        logical.plone.register.direct_login(self._user)

    def scenario(self):
        """
        User registers
        """
        self.expect_ok(1,2,3,4)

    def scenario_3a(self):
        """
        User enters password different from the confirmation password
        """
        password = self._user['password']
        self._user['password']='differentfirstpassword'
        self.expect_ko(1,2,3)
        # Recover from the error by filling in the correct password this time
        self._user['password']=password
        self.expect_ok(2,3,4)

    def scenario_3b(self):
        """
        User enters no password and no confirmation password
        """
        password = self._user['password']
        password_confirm = self._user['password_confirm']
        self._user['password']=''
        self._user['password_confirm']=''
        self.expect_ko(1,2,3)
        # Recover from the error by filling in the correct passwords
        self._user['password']=password
        self._user['password_confirm']=password
        self.expect_ok(2,3,4)

    def scenario_3c(self):
        """
        User registers twice
        """
        # Register with the user, but don't log in directly
        self.expect_ok(1,2,3)
        # Registering again should produce an error
        self.expect_ko(1,2,3)

    def scenario_3d(self):
        """
        User registers with an empty email address
        """
        email = self._user['email']
        self._user['email'] = ''
        # Register with the user, but there is an error because of the empty email
        self.expect_ko(1,2,3)
        self._user['email'] = email
        # Registering again with the correct email should work
        self.expect_ok(2,3,4)

    def scenario_3f(self):
        """
        User registers with an email address without the @ sign
        """
        email = self._user['email']
        # Register with the user, but there is an error because of the wrong email
        self._user['email'] = email.replace('@','')
        self.expect_ko(1,2,3)
        self._user['email'] = email
        # Registering again with the correct email should work
        self.expect_ok(1,2,3,4)

