from funittest import dataprovider
from funittest import logical
from funittest import scenarios
from funittest import models
from funittest import Schema

class RestrictAddableTypes(models.Scenario):
    """
    The user wants to manually restrict what types of content can be added,
    instead of allowing the standard types to be added.
    
    Precondition
    
    1. We are in the context of a folder
    2. Restricting the addable types is possible
    3. Given a list of allowed types
    4. Given a list of secondary types
    
    Main scenario
    
    1. Select type restrictions in add menu
    3. Select the allowed types
    4. Select secondary types
    5. Save
    """

    schema = Schema({"admin":dataprovider.plone.user,
                     "user":dataprovider.plone.user,
                     "context":dataprovider.plone.context,
                     #"addable":dataprovider.plone.addable
                     })

    def setUp(self):
        scenarios.plone.login(user=self._admin)
        self._addable = {'allowed':['Event','Favorite','Topic'],
                         'secondary':['Topic']}
        scenarios.plone.navigateto(context=self._context)

    def step_1(self):
        logical.plone.typerestrictions.access()

    def step_2(self):
        logical.plone.typerestrictions.set_addable(self._addable)
    
    def step_3(self):
        logical.plone.typerestrictions.submit()

    def scenario(self):
        self.expect_ok(1,2,3)

