from funittest import dataprovider
from funittest import models
from funittest import logical
from funittest import Schema

class ReviewContent(models.Scenario):
    """
    Go to the review list to access a content to review a content.
    
    
    Preparation:
    
    1. Choose a user
    2. Choose a content
    
    Precondition:

    1. There is content in the reviewlist Plone site for the user.
    2. The user is assumed to have access to the content.

    Main scenario
    
    1. Access the Plone site through the full_review_list
    2. Go through the next pages until the content is found
    3. Access the content
    """

    schema = Schema({"content":dataprovider.plone.content,})

    def setUp(self):
        pass

    def step_1(self):
        logical.plone.workflow.access_review_list(self._content)

    def step_2(self):
        logical.plone.workflow.browse_review_list(self._content)

    def step_3(self):
        logical.plone.workflow.access_content_to_review(self._content)

    def scenario(self):
        self.expect_ok(1,2,3)

    def scenario_2a(self):
        """
        User looks for a content, but he does not remember the title correctly.
        He browses the whole list, until he realises his error, and looks for 
        the content with the right title.
        """
        content_title = self._content['title']
        self._content['title']='differenttitletheuser'
        self.expect_ko(1,2)
        self._content['title']=content_title
        self.expect_ok(1,2,3)

