import os

from funittest import dataprovider
from funittest import interpreter
from funittest import tests
from funittest import load_product_version
import funittest.preset.plone3

def login():
    return funittest.preset.plone3.login()

def version():
    # Always configured for the latest Plone 2.5.x
    load_product_version("plone", "2.5.5")
    load_product_version("plonepolicy")

def preset():
    # Change this to the password for the admin user
    dataprovider.plone.user.update("admin", {'id':'admin','password':'admin'})
        
    # News Item type name has changed, but we do not want to duplicate
    # the data, so we just set what we expect
    for key in dataprovider.plone.content.keys():
        value = dataprovider.plone.content.get(key)
        if value['type_name'] == "News+Item":
            value['type_name']="News Item"
            dataprovider.plone.content.set(key,value)
    
    # Presets
    dataprovider.plone.content.current = ['news', 'events']
    dataprovider.plone.transition.current = ['publish', 'reject', 'retract', 'submit']
    dataprovider.plone.portalsettings.current = 'Self registration with password choice and visible ids'
    dataprovider.plone.portalsettings.establish_current = True
    dataprovider.plone.sitesettings.current = 'Plone default settings, but visible ids'
    dataprovider.plone.sitesettings.establish_current = True
    #dataprovider.plone.security.current = 'No self registration'
    dataprovider.plone.user.current = ['anon', 'admin', 'manager', 'member', 'managerreviewer', 'reviewer']
    dataprovider.plone.user.establish_current = True
    dataprovider.plone.context.current = ['/', '/events', '/news', '/Members']
    dataprovider.plone.product.current = []
    dataprovider.plone.path.current = ['news', 'events', 'users']
    # We want to have site security and site settings first before establishing
    # other presets.
    dataproviders = [dataprovider.plone.portalsettings]
    for dp in dataprovider.plone.get_models():
        # We exclude product and the models we already added
        if dp.__module__ == dataprovider.plone.product.__module__:
            pass
        elif dp.__module__ == dataprovider.plone.sitesecurity.__module__:
            pass
        elif dp.__module__ == dataprovider.plone.sitesettings.__module__:
            pass
        elif dp.__module__ == dataprovider.plone.portalsettings.__module__:
            pass
        else:
            dataproviders.append(dp)

    dataprovider.plonepolicy.onestateworkflow.current = ["documentfolder",]
    dataprovider.plonepolicy.searchresultsfound.current = ["searches",]

    tests.plone.addablecontent.configure({'root':{'user':'admin',
                                                  'path':'/',
                                                  'types':['News Item', 
                                                           'Image', 
                                                           'Topic', 
                                                           'Link', 
                                                           'File', 
                                                           'Folder', 
                                                           'Document', 
                                                           'Event'],
                                                 },
                                          'events':{'user':'admin',
                                                    'path':'/events',
                                                    'types':['Topic',],
                                                   },
                                         })

    tests.plone.addcontent.configure({'newsitem':{'user':'admin',
                                                  'path':'/',
                                                  'type':'News Item'},
                                      'image':   {'user':'admin',
                                                  'path':'/',
                                                  'type':'Image'},
                                      'topic':   {'user':'admin',
                                                  'path':'/',
                                                  'type':'Topic'},
                                      'link':    {'user':'admin',
                                                  'path':'/',
                                                  'type':'Link'},
                                      'file':    {'user':'admin',
                                                  'path':'/',
                                                  'type':'File'},
                                      'folder':  {'user':'admin',
                                                  'path':'/',
                                                  'type':'Folder'},
                                      'document':{'user':'admin',
                                                  'path':'/',
                                                  'type':'Document'},
                                      'event':   {'user':'admin',
                                                  'path':'/',
                                                  'type':'Event'},
                                      })

    tests.plone.adddeletecontent.configure({'newsitem':{'user':'admin',
                                                        'path':'/',
                                                        'content':'newsitem1'},
                                            'image':   {'user':'admin',
                                                        'path':'/',
                                                        'content':'image1'},
                                            'topic':   {'user':'admin',
                                                        'path':'/',
                                                        'content':'topic1'},
                                            'link':    {'user':'admin',
                                                        'path':'/',
                                                        'content':'link1'},
                                            'file':    {'user':'admin',
                                                        'path':'/',
                                                        'content':'file1'},
                                            'folder':  {'user':'admin',
                                                        'path':'/',
                                                        'content':'folder1'},
                                            'document':{'user':'admin',
                                                        'path':'/',
                                                        'content':'document1'},
                                            'event':   {'user':'admin',
                                                        'path':'/',
                                                        'content':'event1'},
                                            })

    tests.plone.adduser.configure({'admin':{'user':'admin',
                                            'adduserlike':'admin'},
                                   'manager':{'user':'manager',
                                              'adduserlike':'manager'},
                                   'managerreviewer':{'user':'managerreviewer',
                                                      'adduserlike':'managerreviewer'},
                                  },
                                 )

    tests.plone.deleteuser.configure({'admin':{'user':'admin',
                                               'deleteuserlike':'admin'},
                                      'manager':{'user':'manager',
                                                 'deleteuserlike':'manager'},
                                      'managerreviewer':{'user':'managerreviewer',
                                                         'deleteuserlike':'managerreviewer'},
                                     },
                                    )

    tests.plone.installedproducts.configure({'admin':{'user':'admin',
                                             'installed':['Archetypes', 'MimetypesRegistry', 'PortalTransforms', 'kupu', 'CMFPlacefulWorkflow', 'PasswordResetTool']},
                                            },
                                           )

    tests.plone.login.configure({'admin':{'user':'admin',},
                                 'manager':{'user':'manager',},
                                 'managerreviewer':{'user':'managerreviewer',},
                                 'member':{'user':'member',},
                                 'reviewer':{'user':'reviewer',},
                                 },
                               )

    tests.plone.logout.configure({'admin':{'user':'admin',},
                                  'manager':{'user':'manager',},
                                  'managerreviewer':{'user':'managerreviewer',},
                                  'member':{'user':'member',},
                                  'reviewer':{'user':'reviewer',},
                                  },
                                )

    tests.plone.navigateto.configure({'adminnavigatestoroot':{'user':'admin',
                                                              'context':'root'},
                                      'adminnavigatestoevents':{'user':'admin',
                                                                'context':'events'},
                                      'adminnavigatestonews':{'user':'admin',
                                                              'context':'news'},
                                      'adminnavigatestousers':{'user':'admin',
                                                               'context':'users'},
                                     },
                                    )
    
    tests.plone.nestcontent.configure({'admin_nestcontent_descending_path_1':{'user':'admin',
                                                                              'path':'descending_path_1'},
                                       'admin_nestcontent_descending_path_2':{'user':'admin',
                                                                              'path':'descending_path_2'},
                                     },
                                    )

    tests.plone.nestcontentmerge.configure({'nestcontentmerge1':{'admin':'admin',
                                                                 'user':'admin',
                                                                 'context':'root',
                                                                 'path1':'descending_path_1',
                                                                 'path2':'descending_path_2'},
                                           },
                                          )

    tests.plone.positioncontent.configure({'positioncontent1':{'admin':'admin',
                                                               'user':'admin',
                                                               'context':'root',
                                                               'path':'descending_path_3',
                                                               'content':'document-1'},
                                           },
                                          )

    tests.plone.preferences.configure({'admin':{'user':'admin',},
                                       'manager':{'user':'manager',},
                                       'managerreviewer':{'user':'managerreviewer',},
                                       'member':{'user':'member',},
                                       'reviewer':{'user':'reviewer',},
                                       },
                                     )
                                     
    tests.plone.review.configure({'document':{'admin':'admin',
                                              'user':'reviewer',
                                              'content':'document-1'},
                                 })

    tests.plone.searchcontent.configure({'searchcontent1':{'admin':'admin',
                                                           'user':'admin',
                                                           'context':'root',
                                                           'path':'descending_path_3',
                                                           'content':'document-1'},
                                        })

    tests.plone.searchuser.configure({'searchuser1':{'user':'admin',
                                                     'searchuserlike':'admin'},
                                     })


    tests.plone.workflow.configure({'workflow1':{'admin':'admin',
                                                 'user':'admin',
                                                 'context':'root',
                                                 'path':'descending_path_3',
                                                 'content':'document-1',
                                                 'pretransitions':['submit'],
                                                 'transitions':['publish'],                                                                 
                                                },
                                   },
                                  )
                                      
    # Return the dataproviders that should be established if nothing more
    # specific is demanded on the command line
    return dataproviders

if __name__ == '__main__':
    preset()
